from __future__ import annotations

import os
import subprocess
import sys
import sysconfig
from collections.abc import Iterable
from typing import NoReturn

from ._version import version as __version__
from .ninja_syntax import Writer, escape, expand

__all__ = ["BIN_DIR", "DATA", "Writer", "__version__", "escape", "expand", "ninja"]


def __dir__() -> list[str]:
    return __all__


def _get_ninja_dir() -> str:
    ninja_exe = "ninja" + sysconfig.get_config_var("EXE")

    # Default path
    path = os.path.join(sysconfig.get_path("scripts"), ninja_exe)
    if os.path.isfile(path):
        return os.path.dirname(path)

    # User path
    if sys.version_info >= (3, 10):
        user_scheme = sysconfig.get_preferred_scheme("user")
    elif os.name == "nt":
        user_scheme = "nt_user"
    elif sys.platform.startswith("darwin") and getattr(sys, "_framework", None):
        user_scheme = "osx_framework_user"
    else:
        user_scheme = "posix_user"

    path = sysconfig.get_path("scripts", scheme=user_scheme)

    if os.path.isfile(os.path.join(path, ninja_exe)):
        return path

    # Fallback to python location
    path = os.path.dirname(sys.executable)
    if os.path.isfile(os.path.join(path, ninja_exe)):
        return path

    return ""


BIN_DIR = _get_ninja_dir()


def _program(name: str, args: Iterable[str]) -> int:
    cmd = os.path.join(BIN_DIR, name)
    return subprocess.call([cmd, *args], close_fds=False)


def ninja() -> NoReturn:
    raise SystemExit(_program('ninja', sys.argv[1:]))



from collections.abc import Mapping, Sequence
from os import PathLike

def escape_path(word: str) -> str: ...

class Writer:
    output: str
    width: int

    def __init__(self, output: str, width: int = ...): ...
    def newline(self) -> None: ...
    def comment(self, text: str) -> None: ...
    def variable(self, key: str, value: list[str] | str, indent: int = ...) -> None: ...
    def pool(self, name: str, depth: int) -> None: ...
    def rule(self, name: str, command: str, description: str | None = None,
             depfile: str | None = None, generator: bool = False,
             pool: str | None = None, restat: bool = False,
             rspfile: str | None = None, rspfile_content: str | None = None,
             deps: str | None = None) -> None:
        ...

    def build(self, outputs: list[str], rule: str, inputs: list[str] | None = None,
              implicit: list[str] | None = None, order_only: list[str] | None = None,
              variables: dict[str, str] | None = None,
              implicit_outputs: list[str] | None = None,
              pool: str | None = None, dyndep: str | None = None) -> None:
        ...
    def include(self, path: str | PathLike[str]) -> None: ...
    def subninja(self, path: str | PathLike[str]) -> None: ...
    def default(self, paths: Sequence[str | PathLike[str]]) -> None: ...
    def close(self) -> None: ...

def as_list(input: None | list[str] | str) -> list[str]: ...

def escape(string: str) -> str: ...

def expand(string: str, vars: Mapping[str, str], local_vars: Mapping[str, str]=...) -> str: ...



from __future__ import annotations

import os

from scikit_build_core import build as _orig

if hasattr(_orig, "prepare_metadata_for_build_editable"):
    prepare_metadata_for_build_editable = _orig.prepare_metadata_for_build_editable
if hasattr(_orig, "prepare_metadata_for_build_wheel"):
    prepare_metadata_for_build_wheel = _orig.prepare_metadata_for_build_wheel
build_editable = _orig.build_editable
build_wheel = _orig.build_wheel
build_sdist = _orig.build_sdist
get_requires_for_build_editable = _orig.get_requires_for_build_editable
get_requires_for_build_sdist = _orig.get_requires_for_build_sdist

def get_requires_for_build_wheel(config_settings=None):
    packages_orig = _orig.get_requires_for_build_wheel(config_settings)
    if os.environ.get("NINJA_PYTHON_DIST_ALLOW_NINJA_DEP", "0") != "0":
        return packages_orig
    packages = []
    for package in packages_orig:
        package_name = package.lower().split(">")[0].strip()
        if package_name == "ninja":
            # never request ninja from the ninja build
            continue
        packages.append(package)
    return packages



from __future__ import annotations

import os
import subprocess
import sys
import sysconfig

import pytest
from importlib_metadata import distribution

import ninja

from . import push_argv


def _run(program, args):
    func = getattr(ninja, program)
    args = [f"{program}.py", *args]
    with push_argv(args), pytest.raises(SystemExit) as excinfo:
        func()
    assert excinfo.value.code == 0


def _get_scripts():
    dist = distribution("ninja")
    scripts_paths = [os.path.abspath(sysconfig.get_path("scripts", scheme)) for scheme in sysconfig.get_scheme_names()]
    scripts = []
    for file in dist.files:
        if os.path.abspath(str(file.locate().parent)) in scripts_paths:
            scripts.append(file.locate().resolve(strict=True))
    return scripts


def test_ninja_module():
    _run("ninja", ["--version"])


def test_ninja_package():
    expected_version = "1.11.1.git.kitware.jobserver-1"
    output = subprocess.check_output([sys.executable, "-m", "ninja", "--version"]).decode("ascii")
    assert output.splitlines()[0] == expected_version


def test_ninja_script():
    expected_version = "1.11.1.git.kitware.jobserver-1"
    scripts = _get_scripts()
    assert len(scripts) == 1
    assert scripts[0].stem == "ninja"
    output = subprocess.check_output([str(scripts[0]), "--version"]).decode("ascii")
    assert output.splitlines()[0] == expected_version
